"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: d048447bc7bc

from __future__ import annotations
from enum import Enum


class TransferSchemeType(str, Enum):
    r"""The transfer scheme to be used for the transfer. The transfer scheme determines the processing time and method of the transfer."""

    SEPA_CREDIT_INST = "sepa-credit-inst"
    SEPA_CREDIT = "sepa-credit"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: cf97a8e1bc72

from __future__ import annotations
from enum import Enum
from mollie import utils


class VerificationResultEnum(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The result of the Verification of Payee check."""

    MATCH = "match"
    NO_MATCH = "no-match"
    CLOSE_MATCH = "close-match"
    NOT_AVAILABLE = "not-available"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: e6ae99849717

from __future__ import annotations
from .status_reason_code_response import StatusReasonCodeResponse
from mollie import models
from mollie.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
from pydantic import field_serializer, model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class StatusReason2TypedDict(TypedDict):
    r"""A human-readable reason explaining the current status of the transfer. Only populated when the transfer has
    reached a terminal status of `failed`, `rejected`, or `blocked`.

    This field is `null` for transfers that have not reached one of these statuses.
    """

    code: NotRequired[StatusReasonCodeResponse]
    r"""A machine-readable code indicating the reason for the transfer's terminal status."""
    message: NotRequired[Nullable[str]]
    r"""Provides further details about failure indicated. This field is only populated if the`code` field is set to `error`."""


class StatusReason2(BaseModel):
    r"""A human-readable reason explaining the current status of the transfer. Only populated when the transfer has
    reached a terminal status of `failed`, `rejected`, or `blocked`.

    This field is `null` for transfers that have not reached one of these statuses.
    """

    code: Optional[StatusReasonCodeResponse] = None
    r"""A machine-readable code indicating the reason for the transfer's terminal status."""

    message: OptionalNullable[str] = UNSET
    r"""Provides further details about failure indicated. This field is only populated if the`code` field is set to `error`."""

    @field_serializer("code")
    def serialize_code(self, value):
        if isinstance(value, str):
            try:
                return models.StatusReasonCodeResponse(value)
            except ValueError:
                return value
        return value

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["code", "message"])
        nullable_fields = set(["message"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 6ec048477f9b

from __future__ import annotations
from .account_number_format_response import AccountNumberFormatResponse
from mollie import models
from mollie.types import BaseModel
import pydantic
from pydantic import field_serializer
from typing_extensions import Annotated, TypedDict


class CreditorBankAccountResponseTypedDict(TypedDict):
    r"""The bank account details of the creditor (recipient) for Verification of Payee."""

    account_holder_name: str
    r"""The full name of the creditor account holder to verify against bank records."""
    format_: AccountNumberFormatResponse
    r"""The format of the account number."""
    account_number: str
    r"""The bank account details of the creditor."""


class CreditorBankAccountResponse(BaseModel):
    r"""The bank account details of the creditor (recipient) for Verification of Payee."""

    account_holder_name: Annotated[str, pydantic.Field(alias="accountHolderName")]
    r"""The full name of the creditor account holder to verify against bank records."""

    format_: Annotated[AccountNumberFormatResponse, pydantic.Field(alias="format")]
    r"""The format of the account number."""

    account_number: Annotated[str, pydantic.Field(alias="accountNumber")]
    r"""The bank account details of the creditor."""

    @field_serializer("format_")
    def serialize_format_(self, value):
        if isinstance(value, str):
            try:
                return models.AccountNumberFormatResponse(value)
            except ValueError:
                return value
        return value


try:
    CreditorBankAccountResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 4f9d5475e1a5

from __future__ import annotations
from .transfer_request import TransferRequest, TransferRequestTypedDict
from mollie.types import BaseModel, UNSET_SENTINEL
from mollie.utils import FieldMetadata, HeaderMetadata, RequestMetadata
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class CreateTransferRequestTypedDict(TypedDict):
    x_client_signature: str
    r"""A cryptographic signature of the request payload, used to verify the authenticity of the transfer request."""
    x_client_signed_at: str
    r"""The timestamp (in ISO 8601 format) indicating when the client signed the request. Used in conjunction with
    `X-Client-Signature` for request verification.
    """
    idempotency_key: str
    r"""A unique value used to identify this request and prevent duplicate transfers. UUIDv4 is recommended to
    guarantee uniqueness across multiple processes or servers. If two requests are received with the same
    idempotency key, the second request will be discarded.

    View the [public documentation](https://docs.mollie.com/reference/api-idempotency#using-an-idempotency-key)
    to learn more.
    """
    idempotency_key1: NotRequired[str]
    r"""A unique key to ensure idempotent requests. This key should be a UUID v4 string."""
    transfer_request: NotRequired[TransferRequestTypedDict]


class CreateTransferRequest(BaseModel):
    x_client_signature: Annotated[
        str,
        pydantic.Field(alias="X-Client-Signature"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ]
    r"""A cryptographic signature of the request payload, used to verify the authenticity of the transfer request."""

    x_client_signed_at: Annotated[
        str,
        pydantic.Field(alias="X-Client-Signed-At"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ]
    r"""The timestamp (in ISO 8601 format) indicating when the client signed the request. Used in conjunction with
    `X-Client-Signature` for request verification.
    """

    idempotency_key: Annotated[
        str,
        pydantic.Field(alias="Idempotency-Key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ]
    r"""A unique value used to identify this request and prevent duplicate transfers. UUIDv4 is recommended to
    guarantee uniqueness across multiple processes or servers. If two requests are received with the same
    idempotency key, the second request will be discarded.

    View the [public documentation](https://docs.mollie.com/reference/api-idempotency#using-an-idempotency-key)
    to learn more.
    """

    idempotency_key1: Annotated[
        Optional[str],
        pydantic.Field(alias="idempotency-key"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""A unique key to ensure idempotent requests. This key should be a UUID v4 string."""

    transfer_request: Annotated[
        Optional[TransferRequest],
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["idempotency-key1", "transfer-request"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 3fe68b09758c

from __future__ import annotations
from enum import Enum
from mollie import utils


class AccountNumberFormatResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The format of the account number."""

    IBAN = "iban"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 2c424d91d84f

from __future__ import annotations
from enum import Enum
from mollie import utils


class TransferSchemeTypeResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The transfer scheme to be used for the transfer. The transfer scheme determines the processing time and method of the transfer."""

    SEPA_CREDIT_INST = "sepa-credit-inst"
    SEPA_CREDIT = "sepa-credit"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: aeba1d370276

from __future__ import annotations
from .account_number_format import AccountNumberFormat
from mollie.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class CreditorBankAccountTypedDict(TypedDict):
    r"""The bank account details of the creditor (recipient) for Verification of Payee."""

    account_holder_name: str
    r"""The full name of the creditor account holder to verify against bank records."""
    format_: AccountNumberFormat
    r"""The format of the account number."""
    account_number: str
    r"""The bank account details of the creditor."""


class CreditorBankAccount(BaseModel):
    r"""The bank account details of the creditor (recipient) for Verification of Payee."""

    account_holder_name: Annotated[str, pydantic.Field(alias="accountHolderName")]
    r"""The full name of the creditor account holder to verify against bank records."""

    format_: Annotated[AccountNumberFormat, pydantic.Field(alias="format")]
    r"""The format of the account number."""

    account_number: Annotated[str, pydantic.Field(alias="accountNumber")]
    r"""The bank account details of the creditor."""


try:
    CreditorBankAccount.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 08294e489fe8

from __future__ import annotations
from .creditor_bank_account import CreditorBankAccount, CreditorBankAccountTypedDict
from mollie.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing_extensions import Annotated, NotRequired, TypedDict


class VerificationOfPayeeRequestTypedDict(TypedDict):
    r"""The request body for performing a Verification of Payee (VoP) check. VoP allows you to verify the
    account holder name against the records held by the receiving bank before initiating a transfer.
    """

    creditor_bank_account: CreditorBankAccountTypedDict
    r"""The bank account details of the creditor (recipient) for Verification of Payee."""
    testmode: NotRequired[Nullable[bool]]
    r"""Whether to create the entity in test mode or live mode.

    Most API credentials are specifically created for either live mode or test mode, in which case this parameter must
    not be sent. For organization-level credentials such as OAuth access tokens, you can enable test mode by setting
    `testmode` to `true`.
    """


class VerificationOfPayeeRequest(BaseModel):
    r"""The request body for performing a Verification of Payee (VoP) check. VoP allows you to verify the
    account holder name against the records held by the receiving bank before initiating a transfer.
    """

    creditor_bank_account: Annotated[
        CreditorBankAccount, pydantic.Field(alias="creditorBankAccount")
    ]
    r"""The bank account details of the creditor (recipient) for Verification of Payee."""

    testmode: OptionalNullable[bool] = UNSET
    r"""Whether to create the entity in test mode or live mode.

    Most API credentials are specifically created for either live mode or test mode, in which case this parameter must
    not be sent. For organization-level credentials such as OAuth access tokens, you can enable test mode by setting
    `testmode` to `true`.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["testmode"])
        nullable_fields = set(["testmode"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    VerificationOfPayeeRequest.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 250f4cb4d537

from __future__ import annotations
from enum import Enum
from mollie import utils


class SettlementPaymentStatus(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The payment's status. Settlement payments always have a status of `paid`."""

    PAID = "paid"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: b8cfcc17d1ad

from __future__ import annotations
from enum import Enum
from mollie import utils


class StatusReasonCodeResponse(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""A machine-readable code indicating the reason for the transfer's terminal status."""

    INSUFFICIENT_FUNDS = "insufficient-funds"
    REJECTED = "rejected"
    ERROR = "error"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 0040e1e52add

from .basesdk import BaseSDK
from mollie import models, utils
from mollie._hooks import HookContext
from mollie.types import OptionalNullable, UNSET
from mollie.utils import get_security_from_env
from mollie.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional, Union


class TransfersSDK(BaseSDK):
    def create(
        self,
        *,
        x_client_signature: str,
        x_client_signed_at: str,
        idempotency_key: str,
        idempotency_key1: Optional[str] = None,
        transfer_request: Optional[
            Union[models.TransferRequest, models.TransferRequestTypedDict]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TransferResponse:
        r"""Create transfer

        > 🚧 Beta feature
        >
        > This feature is currently in beta testing, and the final specification may still change.

        Create a SEPA Credit Transfer from your Mollie Business Account.

        To initiate a transfer, you must provide the transfer scheme, the amount, the debtor IBAN (your Mollie Business
        Account IBAN), and the creditor (recipient) details.

        Each request must include an `Idempotency-Key` header to prevent duplicate transfers, and must be signed using the
        `X-Client-Signature` and `X-Client-Signed-At` headers.

        ### Simulating transfer scenarios in test mode

        In test mode, you can simulate various transfer scenarios by adjusting the transfer amount. This allows you to
        mimic the typical status progression of a real-world transfer. Note that a transfer's progression will stop once
        it reaches a final status: `blocked`, `failed`, or `processed`.

        | Amount  | Scenario                                           | Webhook sequence                                                                                                                                                   |
        |---------|----------------------------------------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------|
        | `11.00` | Transfer initiated, pending review by Mollie       | `business-account-transfer.requested` → `business-account-transfer.initiated` → `business-account-transfer.pending-review`                                         |
        | `12.00` | Transfer initiated, blocked by Mollie              | `business-account-transfer.requested` → `business-account-transfer.initiated` → `business-account-transfer.pending-review` → `business-account-transfer.blocked`   |
        | `13.00` | Transfer initiated, failed on scheme submission    | `business-account-transfer.requested` → `business-account-transfer.initiated` → `business-account-transfer.failed`                                                 |
        | Other   | Default: transfer is processed                     | `business-account-transfer.requested` → `business-account-transfer.initiated` → `business-account-transfer.processed`                                              |

        If set, this operation will use `organization_access_token` from the global security.

        :param x_client_signature: A cryptographic signature of the request payload, used to verify the authenticity of the transfer request.
        :param x_client_signed_at: The timestamp (in ISO 8601 format) indicating when the client signed the request. Used in conjunction with
            `X-Client-Signature` for request verification.
        :param idempotency_key: A unique value used to identify this request and prevent duplicate transfers. UUIDv4 is recommended to
            guarantee uniqueness across multiple processes or servers. If two requests are received with the same
            idempotency key, the second request will be discarded.

            View the [public documentation](https://docs.mollie.com/reference/api-idempotency#using-an-idempotency-key)
            to learn more.
        :param idempotency_key1: A unique key to ensure idempotent requests. This key should be a UUID v4 string.
        :param transfer_request:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateTransferRequest(
            x_client_signature=x_client_signature,
            x_client_signed_at=x_client_signed_at,
            idempotency_key=idempotency_key,
            idempotency_key1=idempotency_key1,
            transfer_request=utils.get_pydantic_model(
                transfer_request, Optional[models.TransferRequest]
            ),
        )

        req = self._build_request(
            method="POST",
            path="/business-accounts/transfers",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/hal+json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.transfer_request if request is not None else None,
                False,
                True,
                "json",
                Optional[models.TransferRequest],
            ),
            allow_empty_value=None,
            allowed_fields=["organization_access_token"],
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 5000, 2, 7500), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5xx"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create-transfer",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "503", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/hal+json"):
            return unmarshal_json_response(models.TransferResponse, http_res)
        if utils.match_response(http_res, "422", "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "503", "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        x_client_signature: str,
        x_client_signed_at: str,
        idempotency_key: str,
        idempotency_key1: Optional[str] = None,
        transfer_request: Optional[
            Union[models.TransferRequest, models.TransferRequestTypedDict]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TransferResponse:
        r"""Create transfer

        > 🚧 Beta feature
        >
        > This feature is currently in beta testing, and the final specification may still change.

        Create a SEPA Credit Transfer from your Mollie Business Account.

        To initiate a transfer, you must provide the transfer scheme, the amount, the debtor IBAN (your Mollie Business
        Account IBAN), and the creditor (recipient) details.

        Each request must include an `Idempotency-Key` header to prevent duplicate transfers, and must be signed using the
        `X-Client-Signature` and `X-Client-Signed-At` headers.

        ### Simulating transfer scenarios in test mode

        In test mode, you can simulate various transfer scenarios by adjusting the transfer amount. This allows you to
        mimic the typical status progression of a real-world transfer. Note that a transfer's progression will stop once
        it reaches a final status: `blocked`, `failed`, or `processed`.

        | Amount  | Scenario                                           | Webhook sequence                                                                                                                                                   |
        |---------|----------------------------------------------------|--------------------------------------------------------------------------------------------------------------------------------------------------------------------|
        | `11.00` | Transfer initiated, pending review by Mollie       | `business-account-transfer.requested` → `business-account-transfer.initiated` → `business-account-transfer.pending-review`                                         |
        | `12.00` | Transfer initiated, blocked by Mollie              | `business-account-transfer.requested` → `business-account-transfer.initiated` → `business-account-transfer.pending-review` → `business-account-transfer.blocked`   |
        | `13.00` | Transfer initiated, failed on scheme submission    | `business-account-transfer.requested` → `business-account-transfer.initiated` → `business-account-transfer.failed`                                                 |
        | Other   | Default: transfer is processed                     | `business-account-transfer.requested` → `business-account-transfer.initiated` → `business-account-transfer.processed`                                              |

        If set, this operation will use `organization_access_token` from the global security.

        :param x_client_signature: A cryptographic signature of the request payload, used to verify the authenticity of the transfer request.
        :param x_client_signed_at: The timestamp (in ISO 8601 format) indicating when the client signed the request. Used in conjunction with
            `X-Client-Signature` for request verification.
        :param idempotency_key: A unique value used to identify this request and prevent duplicate transfers. UUIDv4 is recommended to
            guarantee uniqueness across multiple processes or servers. If two requests are received with the same
            idempotency key, the second request will be discarded.

            View the [public documentation](https://docs.mollie.com/reference/api-idempotency#using-an-idempotency-key)
            to learn more.
        :param idempotency_key1: A unique key to ensure idempotent requests. This key should be a UUID v4 string.
        :param transfer_request:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateTransferRequest(
            x_client_signature=x_client_signature,
            x_client_signed_at=x_client_signed_at,
            idempotency_key=idempotency_key,
            idempotency_key1=idempotency_key1,
            transfer_request=utils.get_pydantic_model(
                transfer_request, Optional[models.TransferRequest]
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/business-accounts/transfers",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/hal+json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.transfer_request if request is not None else None,
                False,
                True,
                "json",
                Optional[models.TransferRequest],
            ),
            allow_empty_value=None,
            allowed_fields=["organization_access_token"],
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 5000, 2, 7500), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5xx"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create-transfer",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "503", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/hal+json"):
            return unmarshal_json_response(models.TransferResponse, http_res)
        if utils.match_response(http_res, "422", "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "503", "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def get(
        self,
        *,
        business_accounts_transfer_id: str,
        testmode: Optional[bool] = None,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TransferResponse:
        r"""Get transfer

        > 🚧 Beta feature
        >
        > This feature is currently in beta testing, and the final specification may still change.

        Retrieve a single transfer object by its transfer ID. This allows you to check the current status
        and details of a previously created transfer.

        If set, this operation will use `organization_access_token` from the global security.

        :param business_accounts_transfer_id: Provide the ID of the related transfer.
        :param testmode: Most API credentials are specifically created for either live mode or test mode. In those cases the `testmode` query
            parameter must not be sent. For organization-level credentials such as OAuth access tokens, you can enable test mode by
            setting the `testmode` query parameter to `true`.

            Test entities cannot be retrieved when the endpoint is set to live mode, and vice versa.
        :param idempotency_key: A unique key to ensure idempotent requests. This key should be a UUID v4 string.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetTransferRequest(
            business_accounts_transfer_id=business_accounts_transfer_id,
            testmode=testmode,
            idempotency_key=idempotency_key,
        )

        req = self._build_request(
            method="GET",
            path="/business-accounts/transfers/{businessAccountsTransferId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/hal+json",
            http_headers=http_headers,
            _globals=models.GetTransferGlobals(
                testmode=self.sdk_configuration.globals.testmode,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            allowed_fields=["organization_access_token"],
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 5000, 2, 7500), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5xx"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get-transfer",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/hal+json"):
            return unmarshal_json_response(models.TransferResponse, http_res)
        if utils.match_response(http_res, "404", "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        business_accounts_transfer_id: str,
        testmode: Optional[bool] = None,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.TransferResponse:
        r"""Get transfer

        > 🚧 Beta feature
        >
        > This feature is currently in beta testing, and the final specification may still change.

        Retrieve a single transfer object by its transfer ID. This allows you to check the current status
        and details of a previously created transfer.

        If set, this operation will use `organization_access_token` from the global security.

        :param business_accounts_transfer_id: Provide the ID of the related transfer.
        :param testmode: Most API credentials are specifically created for either live mode or test mode. In those cases the `testmode` query
            parameter must not be sent. For organization-level credentials such as OAuth access tokens, you can enable test mode by
            setting the `testmode` query parameter to `true`.

            Test entities cannot be retrieved when the endpoint is set to live mode, and vice versa.
        :param idempotency_key: A unique key to ensure idempotent requests. This key should be a UUID v4 string.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetTransferRequest(
            business_accounts_transfer_id=business_accounts_transfer_id,
            testmode=testmode,
            idempotency_key=idempotency_key,
        )

        req = self._build_request_async(
            method="GET",
            path="/business-accounts/transfers/{businessAccountsTransferId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/hal+json",
            http_headers=http_headers,
            _globals=models.GetTransferGlobals(
                testmode=self.sdk_configuration.globals.testmode,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            allowed_fields=["organization_access_token"],
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 5000, 2, 7500), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5xx"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get-transfer",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/hal+json"):
            return unmarshal_json_response(models.TransferResponse, http_res)
        if utils.match_response(http_res, "404", "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: a25c5299fa39

from __future__ import annotations
from .amount import Amount, AmountTypedDict
from .credit_debit_indicator import CreditDebitIndicator
from .metadata import Metadata, MetadataTypedDict
from .mode import Mode
from .status_history_entry_response import (
    StatusHistoryEntryResponse,
    StatusHistoryEntryResponseTypedDict,
)
from .status_reason_2 import StatusReason2, StatusReason2TypedDict
from .transfer_party import TransferParty, TransferPartyTypedDict
from .transfer_scheme_response import (
    TransferSchemeResponse,
    TransferSchemeResponseTypedDict,
)
from .transfer_status import TransferStatus
from mollie import models
from mollie.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import field_serializer, model_serializer
from typing import List
from typing_extensions import Annotated, NotRequired, TypedDict


class TransferResponseAccountTypedDict(TypedDict):
    r"""The bank account details of the party."""

    iban: str
    r"""The IBAN (International Bank Account Number) of the account holder."""


class TransferResponseAccount(BaseModel):
    r"""The bank account details of the party."""

    iban: str
    r"""The IBAN (International Bank Account Number) of the account holder."""


class DebtorTypedDict(TypedDict):
    r"""The debtor (sender) of the transfer, including their name and account details."""

    full_name: str
    r"""The full name of the account holder."""
    account: TransferResponseAccountTypedDict
    r"""The bank account details of the party."""


class Debtor(BaseModel):
    r"""The debtor (sender) of the transfer, including their name and account details."""

    full_name: Annotated[str, pydantic.Field(alias="fullName")]
    r"""The full name of the account holder."""

    account: TransferResponseAccount
    r"""The bank account details of the party."""


class TransferResponseTypedDict(TypedDict):
    resource: str
    r"""Indicates the response contains a transfer object. Will always contain the string `business-account-transfer`
    for this endpoint.
    """
    id: str
    r"""The identifier uniquely referring to this transfer. Mollie assigns this identifier at transfer creation time."""
    mode: Mode
    r"""Whether this entity was created in live mode or in test mode."""
    debtor: DebtorTypedDict
    r"""The debtor (sender) of the transfer, including their name and account details."""
    creditor: TransferPartyTypedDict
    r"""A party involved in the transfer, representing either the debtor (sender) or creditor (recipient).
    Contains the party's name and account details.
    """
    amount: AmountTypedDict
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""
    business_account_transaction_id: str
    r"""The identifier of the corresponding transaction in the business accounts transaction resource."""
    transfer_scheme: TransferSchemeResponseTypedDict
    r"""The scheme, as requested by the client."""
    credit_debit_indicator: CreditDebitIndicator
    r"""Indicates whether the transfer is a credit or debit transaction from the perspective of the account holder."""
    status: TransferStatus
    r"""The status of the transfer."""
    status_history: List[StatusHistoryEntryResponseTypedDict]
    r"""A chronological list of status transitions the transfer has gone through."""
    created_at: str
    r"""The entity's date and time of creation, in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format."""
    description: NotRequired[Nullable[str]]
    r"""A short description of the transfer. This will appear on the bank statement of both the debtor and creditor.

    It must begin with an alphanumeric character, followed by any combination of letters, numbers, spaces or special characters `/ - ? : ( ) . , ' + _`.

    Constraints:
    - Cannot be solely a hyphen (-) or colon (:)
    - Cannot contain consecutive hyphens (--)
    - Cannot contain consecutive colons (::)
    """
    status_reason: NotRequired[Nullable[StatusReason2TypedDict]]
    r"""A human-readable reason explaining the current status of the transfer. Only populated when the transfer has
    reached a terminal status of `failed`, `rejected`, or `blocked`.

    This field is `null` for transfers that have not reached one of these statuses.
    """
    metadata: NotRequired[Nullable[MetadataTypedDict]]
    r"""Provide any data you like, for example a string or a JSON object. We will save the data alongside the entity. Whenever
    you fetch the entity with our API, we will also include the metadata. You can use up to approximately 1kB.
    """


class TransferResponse(BaseModel):
    resource: str
    r"""Indicates the response contains a transfer object. Will always contain the string `business-account-transfer`
    for this endpoint.
    """

    id: str
    r"""The identifier uniquely referring to this transfer. Mollie assigns this identifier at transfer creation time."""

    mode: Mode
    r"""Whether this entity was created in live mode or in test mode."""

    debtor: Debtor
    r"""The debtor (sender) of the transfer, including their name and account details."""

    creditor: TransferParty
    r"""A party involved in the transfer, representing either the debtor (sender) or creditor (recipient).
    Contains the party's name and account details.
    """

    amount: Amount
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""

    business_account_transaction_id: Annotated[
        str, pydantic.Field(alias="businessAccountTransactionId")
    ]
    r"""The identifier of the corresponding transaction in the business accounts transaction resource."""

    transfer_scheme: Annotated[
        TransferSchemeResponse, pydantic.Field(alias="transferScheme")
    ]
    r"""The scheme, as requested by the client."""

    credit_debit_indicator: Annotated[
        CreditDebitIndicator, pydantic.Field(alias="creditDebitIndicator")
    ]
    r"""Indicates whether the transfer is a credit or debit transaction from the perspective of the account holder."""

    status: TransferStatus
    r"""The status of the transfer."""

    status_history: Annotated[
        List[StatusHistoryEntryResponse], pydantic.Field(alias="statusHistory")
    ]
    r"""A chronological list of status transitions the transfer has gone through."""

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""The entity's date and time of creation, in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format."""

    description: OptionalNullable[str] = UNSET
    r"""A short description of the transfer. This will appear on the bank statement of both the debtor and creditor.

    It must begin with an alphanumeric character, followed by any combination of letters, numbers, spaces or special characters `/ - ? : ( ) . , ' + _`.

    Constraints:
    - Cannot be solely a hyphen (-) or colon (:)
    - Cannot contain consecutive hyphens (--)
    - Cannot contain consecutive colons (::)
    """

    status_reason: Annotated[
        OptionalNullable[StatusReason2], pydantic.Field(alias="statusReason")
    ] = UNSET
    r"""A human-readable reason explaining the current status of the transfer. Only populated when the transfer has
    reached a terminal status of `failed`, `rejected`, or `blocked`.

    This field is `null` for transfers that have not reached one of these statuses.
    """

    metadata: OptionalNullable[Metadata] = UNSET
    r"""Provide any data you like, for example a string or a JSON object. We will save the data alongside the entity. Whenever
    you fetch the entity with our API, we will also include the metadata. You can use up to approximately 1kB.
    """

    @field_serializer("mode")
    def serialize_mode(self, value):
        if isinstance(value, str):
            try:
                return models.Mode(value)
            except ValueError:
                return value
        return value

    @field_serializer("credit_debit_indicator")
    def serialize_credit_debit_indicator(self, value):
        if isinstance(value, str):
            try:
                return models.CreditDebitIndicator(value)
            except ValueError:
                return value
        return value

    @field_serializer("status")
    def serialize_status(self, value):
        if isinstance(value, str):
            try:
                return models.TransferStatus(value)
            except ValueError:
                return value
        return value

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "statusReason", "metadata"])
        nullable_fields = set(["description", "statusReason", "metadata"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    Debtor.model_rebuild()
except NameError:
    pass
try:
    TransferResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 8971386eae33

from __future__ import annotations
from enum import Enum


class AccountNumberFormat(str, Enum):
    r"""The format of the account number."""

    IBAN = "iban"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: cb8855e6c849

from __future__ import annotations
from .status_reason_2 import StatusReason2, StatusReason2TypedDict
from .transfer_status import TransferStatus
from mollie import models
from mollie.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import field_serializer, model_serializer
from typing_extensions import Annotated, NotRequired, TypedDict


class StatusHistoryEntryResponseTypedDict(TypedDict):
    r"""Represents an entry in the transfer's status history, recording a past status transition."""

    status: TransferStatus
    r"""The status of the transfer."""
    created_at: str
    r"""The entity's date and time of creation, in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format."""
    status_reason: NotRequired[Nullable[StatusReason2TypedDict]]
    r"""A human-readable reason explaining the current status of the transfer. Only populated when the transfer has
    reached a terminal status of `failed`, `rejected`, or `blocked`.

    This field is `null` for transfers that have not reached one of these statuses.
    """


class StatusHistoryEntryResponse(BaseModel):
    r"""Represents an entry in the transfer's status history, recording a past status transition."""

    status: TransferStatus
    r"""The status of the transfer."""

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""The entity's date and time of creation, in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format."""

    status_reason: Annotated[
        OptionalNullable[StatusReason2], pydantic.Field(alias="statusReason")
    ] = UNSET
    r"""A human-readable reason explaining the current status of the transfer. Only populated when the transfer has
    reached a terminal status of `failed`, `rejected`, or `blocked`.

    This field is `null` for transfers that have not reached one of these statuses.
    """

    @field_serializer("status")
    def serialize_status(self, value):
        if isinstance(value, str):
            try:
                return models.TransferStatus(value)
            except ValueError:
                return value
        return value

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["statusReason"])
        nullable_fields = set(["statusReason"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    StatusHistoryEntryResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: f56d0df6ccff

from .basesdk import BaseSDK
from mollie import models, utils
from mollie._hooks import HookContext
from mollie.types import OptionalNullable, UNSET
from mollie.utils import get_security_from_env
from mollie.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Mapping, Optional, Union


class VerifyPayees(BaseSDK):
    def create(
        self,
        *,
        idempotency_key: Optional[str] = None,
        verification_of_payee_request: Optional[
            Union[
                models.VerificationOfPayeeRequest,
                models.VerificationOfPayeeRequestTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.VerificationOfPayeeResponse:
        r"""Verify Payee

        > 🚧 Beta feature
        >
        > This feature is currently in beta testing, and the final specification may still change.

        Perform a Verification of Payee (VoP) check. This allows you to verify the account holder name against the
        records held by the receiving bank before initiating a transfer.

        The verification result indicates whether the provided name matches, closely matches, or does not match the
        name on file at the receiving bank. This helps prevent misdirected payments.

        ### Simulating verification scenarios in test mode

        In test mode, you can simulate various verification outcomes by adjusting the creditor name in the
        `creditorBankAccount.accountHolderName` property. This allows you to test all possible Verification of Payee
        results without needing special properties. The names are case insensitive.

        | Account holder name                    | Scenario                                      | Verification result | Suggested name |
        |----------------------------------------|-----------------------------------------------|---------------------|----------------|
        | `John Close Match`                     | Name closely matches the bank records          | `close-match`       | `John Match`   |
        | `John No Match`                        | Name does not match the bank records           | `no-match`          | —              |
        | `John Unavailable`                     | Verification is not available                  | `not-available`     | —              |
        | Any other name                         | Default: name matches the bank records         | `match`             | —              |

        If set, this operation will use `organization_access_token` from the global security.

        :param idempotency_key: A unique key to ensure idempotent requests. This key should be a UUID v4 string.
        :param verification_of_payee_request:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.VerifyPayeeRequest(
            idempotency_key=idempotency_key,
            verification_of_payee_request=utils.get_pydantic_model(
                verification_of_payee_request,
                Optional[models.VerificationOfPayeeRequest],
            ),
        )

        req = self._build_request(
            method="POST",
            path="/business-accounts/payee-verifications",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/hal+json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.verification_of_payee_request if request is not None else None,
                False,
                True,
                "json",
                Optional[models.VerificationOfPayeeRequest],
            ),
            allow_empty_value=None,
            allowed_fields=["organization_access_token"],
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 5000, 2, 7500), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5xx"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="verify-payee",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "429", "4XX", "503", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/hal+json"):
            return unmarshal_json_response(models.VerificationOfPayeeResponse, http_res)
        if utils.match_response(http_res, ["422", "429"], "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "503", "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        idempotency_key: Optional[str] = None,
        verification_of_payee_request: Optional[
            Union[
                models.VerificationOfPayeeRequest,
                models.VerificationOfPayeeRequestTypedDict,
            ]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.VerificationOfPayeeResponse:
        r"""Verify Payee

        > 🚧 Beta feature
        >
        > This feature is currently in beta testing, and the final specification may still change.

        Perform a Verification of Payee (VoP) check. This allows you to verify the account holder name against the
        records held by the receiving bank before initiating a transfer.

        The verification result indicates whether the provided name matches, closely matches, or does not match the
        name on file at the receiving bank. This helps prevent misdirected payments.

        ### Simulating verification scenarios in test mode

        In test mode, you can simulate various verification outcomes by adjusting the creditor name in the
        `creditorBankAccount.accountHolderName` property. This allows you to test all possible Verification of Payee
        results without needing special properties. The names are case insensitive.

        | Account holder name                    | Scenario                                      | Verification result | Suggested name |
        |----------------------------------------|-----------------------------------------------|---------------------|----------------|
        | `John Close Match`                     | Name closely matches the bank records          | `close-match`       | `John Match`   |
        | `John No Match`                        | Name does not match the bank records           | `no-match`          | —              |
        | `John Unavailable`                     | Verification is not available                  | `not-available`     | —              |
        | Any other name                         | Default: name matches the bank records         | `match`             | —              |

        If set, this operation will use `organization_access_token` from the global security.

        :param idempotency_key: A unique key to ensure idempotent requests. This key should be a UUID v4 string.
        :param verification_of_payee_request:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.VerifyPayeeRequest(
            idempotency_key=idempotency_key,
            verification_of_payee_request=utils.get_pydantic_model(
                verification_of_payee_request,
                Optional[models.VerificationOfPayeeRequest],
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/business-accounts/payee-verifications",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/hal+json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.verification_of_payee_request if request is not None else None,
                False,
                True,
                "json",
                Optional[models.VerificationOfPayeeRequest],
            ),
            allow_empty_value=None,
            allowed_fields=["organization_access_token"],
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 5000, 2, 7500), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5xx"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="verify-payee",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "429", "4XX", "503", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/hal+json"):
            return unmarshal_json_response(models.VerificationOfPayeeResponse, http_res)
        if utils.match_response(http_res, ["422", "429"], "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "503", "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: ae6e0af73d2b

from __future__ import annotations
from .creditor_bank_account_response import (
    CreditorBankAccountResponse,
    CreditorBankAccountResponseTypedDict,
)
from .verification_result_enum import VerificationResultEnum
from mollie import models
from mollie.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import field_serializer, model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class VerificationOfPayeeResponseVerificationResultTypedDict(TypedDict):
    outcome: VerificationResultEnum
    r"""The result of the Verification of Payee check."""
    account_holder_name: NotRequired[str]
    r"""The corrected name if the verification result is `close_match`."""


class VerificationOfPayeeResponseVerificationResult(BaseModel):
    outcome: VerificationResultEnum
    r"""The result of the Verification of Payee check."""

    account_holder_name: Annotated[
        Optional[str], pydantic.Field(alias="accountHolderName")
    ] = None
    r"""The corrected name if the verification result is `close_match`."""

    @field_serializer("outcome")
    def serialize_outcome(self, value):
        if isinstance(value, str):
            try:
                return models.VerificationResultEnum(value)
            except ValueError:
                return value
        return value

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["accountHolderName"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class VerificationOfPayeeResponseTypedDict(TypedDict):
    r"""The Verification of Payee response object. Contains the result of verifying the creditor's
    account holder name against the records held by the receiving bank.
    """

    resource: str
    r"""Indicates the response contains a payee verification object. Will always contain the string
    `business-account-payee-verification` for this endpoint.
    """
    creditor_bank_account: CreditorBankAccountResponseTypedDict
    r"""The bank account details of the creditor (recipient) for Verification of Payee."""
    verification_result: VerificationOfPayeeResponseVerificationResultTypedDict
    created_at: str
    r"""The entity's date and time of creation, in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format."""


class VerificationOfPayeeResponse(BaseModel):
    r"""The Verification of Payee response object. Contains the result of verifying the creditor's
    account holder name against the records held by the receiving bank.
    """

    resource: str
    r"""Indicates the response contains a payee verification object. Will always contain the string
    `business-account-payee-verification` for this endpoint.
    """

    creditor_bank_account: Annotated[
        CreditorBankAccountResponse, pydantic.Field(alias="creditorBankAccount")
    ]
    r"""The bank account details of the creditor (recipient) for Verification of Payee."""

    verification_result: Annotated[
        VerificationOfPayeeResponseVerificationResult,
        pydantic.Field(alias="verificationResult"),
    ]

    created_at: Annotated[str, pydantic.Field(alias="createdAt")]
    r"""The entity's date and time of creation, in [ISO 8601](https://en.wikipedia.org/wiki/ISO_8601) format."""


try:
    VerificationOfPayeeResponseVerificationResult.model_rebuild()
except NameError:
    pass
try:
    VerificationOfPayeeResponse.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 29e01828928a

from __future__ import annotations
from .transfer_scheme_type import TransferSchemeType
from mollie.types import BaseModel
from typing_extensions import TypedDict


class TransferSchemeTypedDict(TypedDict):
    r"""The scheme, as requested by the client."""

    type: TransferSchemeType
    r"""The transfer scheme to be used for the transfer. The transfer scheme determines the processing time and method of the transfer."""


class TransferScheme(BaseModel):
    r"""The scheme, as requested by the client."""

    type: TransferSchemeType
    r"""The transfer scheme to be used for the transfer. The transfer scheme determines the processing time and method of the transfer."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: fd621abe5fb7

from .basesdk import BaseSDK
from jsonpath import JSONPath
from mollie import models, utils
from mollie._hooks import HookContext
from mollie.types import OptionalNullable, UNSET
from mollie.utils import get_security_from_env
from mollie.utils.unmarshal_json_response import unmarshal_json_response
from typing import Any, Awaitable, Dict, List, Mapping, Optional, Union


class BalanceTransfers(BaseSDK):
    def create(
        self,
        *,
        idempotency_key: Optional[str] = None,
        entity_balance_transfer: Optional[
            Union[models.EntityBalanceTransfer, models.EntityBalanceTransferTypedDict]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EntityBalanceTransferResponse:
        r"""Create a Connect balance transfer

        This API endpoint allows you to create a balance transfer from your organization's balance to a connected organization's balance, or vice versa.
        You can also create a balance transfer between two connected organizations.
        To create a balance transfer, you must be authenticated as the source organization, and the destination organization must be a connected organization
        that has authorized the `balance-transfers.write` scope for your organization.

        If set, this operation will use either `organization_access_token` or `o_auth` from the global security.

        :param idempotency_key: A unique key to ensure idempotent requests. This key should be a UUID v4 string.
        :param entity_balance_transfer:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateConnectBalanceTransferRequest(
            idempotency_key=idempotency_key,
            entity_balance_transfer=utils.get_pydantic_model(
                entity_balance_transfer, Optional[models.EntityBalanceTransfer]
            ),
        )

        req = self._build_request(
            method="POST",
            path="/connect/balance-transfers",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/hal+json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.entity_balance_transfer if request is not None else None,
                False,
                True,
                "json",
                Optional[models.EntityBalanceTransfer],
            ),
            allow_empty_value=None,
            allowed_fields=["organization_access_token", "o_auth"],
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 5000, 2, 7500), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5xx"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create-connect-balance-transfer",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/hal+json"):
            return unmarshal_json_response(
                models.EntityBalanceTransferResponse, http_res
            )
        if utils.match_response(http_res, "422", "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def create_async(
        self,
        *,
        idempotency_key: Optional[str] = None,
        entity_balance_transfer: Optional[
            Union[models.EntityBalanceTransfer, models.EntityBalanceTransferTypedDict]
        ] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EntityBalanceTransferResponse:
        r"""Create a Connect balance transfer

        This API endpoint allows you to create a balance transfer from your organization's balance to a connected organization's balance, or vice versa.
        You can also create a balance transfer between two connected organizations.
        To create a balance transfer, you must be authenticated as the source organization, and the destination organization must be a connected organization
        that has authorized the `balance-transfers.write` scope for your organization.

        If set, this operation will use either `organization_access_token` or `o_auth` from the global security.

        :param idempotency_key: A unique key to ensure idempotent requests. This key should be a UUID v4 string.
        :param entity_balance_transfer:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.CreateConnectBalanceTransferRequest(
            idempotency_key=idempotency_key,
            entity_balance_transfer=utils.get_pydantic_model(
                entity_balance_transfer, Optional[models.EntityBalanceTransfer]
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/connect/balance-transfers",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/hal+json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.entity_balance_transfer if request is not None else None,
                False,
                True,
                "json",
                Optional[models.EntityBalanceTransfer],
            ),
            allow_empty_value=None,
            allowed_fields=["organization_access_token", "o_auth"],
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 5000, 2, 7500), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5xx"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="create-connect-balance-transfer",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "201", "application/hal+json"):
            return unmarshal_json_response(
                models.EntityBalanceTransferResponse, http_res
            )
        if utils.match_response(http_res, "422", "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def list(
        self,
        *,
        from_: OptionalNullable[str] = UNSET,
        limit: OptionalNullable[int] = UNSET,
        sort: Optional[models.Sorting] = None,
        testmode: Optional[bool] = None,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        url_override: Optional[str] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.ListConnectBalanceTransfersResponse]:
        r"""List all Connect balance transfers

        Returns a paginated list of balance transfers associated with your organization. These may be a balance transfer that was received or sent from your balance, or a balance transfer that you initiated on behalf of your clients. If no balance transfers are available, the resulting array will be empty. This request should never throw an error.

        If set, this operation will use either `organization_access_token` or `o_auth` from the global security.

        :param from_: Provide an ID to start the result set from the item with the given ID and onwards. This allows you to paginate the
            result set.
        :param limit: The maximum number of items to return. Defaults to 50 items.
        :param sort: Used for setting the direction of the result set. Defaults to descending order, meaning the results are ordered from
            newest to oldest.
        :param testmode: You can enable test mode by setting the `testmode` query parameter to `true`.

            Test entities cannot be retrieved when the endpoint is set to live mode, and vice versa.
        :param idempotency_key: A unique key to ensure idempotent requests. This key should be a UUID v4 string.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListConnectBalanceTransfersRequest(
            from_=from_,
            limit=limit,
            sort=sort,
            testmode=testmode,
            idempotency_key=idempotency_key,
        )

        req = self._build_request(
            method="GET",
            path="/connect/balance-transfers",
            base_url=base_url,
            url_variables=url_variables,
            url_override=url_override,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/hal+json",
            http_headers=http_headers,
            _globals=models.ListConnectBalanceTransfersGlobals(
                testmode=self.sdk_configuration.globals.testmode,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            allowed_fields=["organization_access_token", "o_auth"],
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 5000, 2, 7500), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5xx"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="list-connect-balance-transfers",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> Optional[models.ListConnectBalanceTransfersResponse]:
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])

            next_url_match = JSONPath("$._links.next.href").parse(body)
            if len(next_url_match) == 0:
                return None

            next_url = next_url_match[0]
            if not next_url:
                return None

            if next_url[0] == "/":
                url = self._get_url(base_url, url_variables)
                next_url = url + next_url

            return self.list(
                from_=from_,
                limit=limit,
                sort=sort,
                testmode=testmode,
                idempotency_key=idempotency_key,
                retries=retries,
                url_override=next_url,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/hal+json"):
            return models.ListConnectBalanceTransfersResponse(
                result=unmarshal_json_response(
                    models.ListConnectBalanceTransfersResponseBody, http_res
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def list_async(
        self,
        *,
        from_: OptionalNullable[str] = UNSET,
        limit: OptionalNullable[int] = UNSET,
        sort: Optional[models.Sorting] = None,
        testmode: Optional[bool] = None,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        url_override: Optional[str] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> Optional[models.ListConnectBalanceTransfersResponse]:
        r"""List all Connect balance transfers

        Returns a paginated list of balance transfers associated with your organization. These may be a balance transfer that was received or sent from your balance, or a balance transfer that you initiated on behalf of your clients. If no balance transfers are available, the resulting array will be empty. This request should never throw an error.

        If set, this operation will use either `organization_access_token` or `o_auth` from the global security.

        :param from_: Provide an ID to start the result set from the item with the given ID and onwards. This allows you to paginate the
            result set.
        :param limit: The maximum number of items to return. Defaults to 50 items.
        :param sort: Used for setting the direction of the result set. Defaults to descending order, meaning the results are ordered from
            newest to oldest.
        :param testmode: You can enable test mode by setting the `testmode` query parameter to `true`.

            Test entities cannot be retrieved when the endpoint is set to live mode, and vice versa.
        :param idempotency_key: A unique key to ensure idempotent requests. This key should be a UUID v4 string.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.ListConnectBalanceTransfersRequest(
            from_=from_,
            limit=limit,
            sort=sort,
            testmode=testmode,
            idempotency_key=idempotency_key,
        )

        req = self._build_request_async(
            method="GET",
            path="/connect/balance-transfers",
            base_url=base_url,
            url_variables=url_variables,
            url_override=url_override,
            request=request,
            request_body_required=False,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/hal+json",
            http_headers=http_headers,
            _globals=models.ListConnectBalanceTransfersGlobals(
                testmode=self.sdk_configuration.globals.testmode,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            allowed_fields=["organization_access_token", "o_auth"],
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 5000, 2, 7500), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5xx"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="list-connect-balance-transfers",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["400", "4XX", "5XX"],
            retry_config=retry_config,
        )

        def next_func() -> (
            Awaitable[Optional[models.ListConnectBalanceTransfersResponse]]
        ):
            body = utils.unmarshal_json(http_res.text, Union[Dict[Any, Any], List[Any]])

            async def empty_result():
                return None

            next_url_match = JSONPath("$._links.next.href").parse(body)
            if len(next_url_match) == 0:
                return empty_result()

            next_url = next_url_match[0]
            if not next_url:
                return empty_result()

            if next_url[0] == "/":
                url = self._get_url(base_url, url_variables)
                next_url = url + next_url

            return self.list_async(
                from_=from_,
                limit=limit,
                sort=sort,
                testmode=testmode,
                idempotency_key=idempotency_key,
                retries=retries,
                url_override=next_url,
            )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/hal+json"):
            return models.ListConnectBalanceTransfersResponse(
                result=unmarshal_json_response(
                    models.ListConnectBalanceTransfersResponseBody, http_res
                ),
                next=next_func,
            )
        if utils.match_response(http_res, "400", "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    def get(
        self,
        *,
        balance_transfer_id: str,
        testmode: Optional[bool] = None,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EntityBalanceTransferResponse:
        r"""Get a Connect balance transfer

        Retrieve a single Connect balance transfer object by its ID.

        If set, this operation will use either `organization_access_token` or `o_auth` from the global security.

        :param balance_transfer_id: Provide the ID of the related balance transfer.
        :param testmode: You can enable test mode by setting the `testmode` query parameter to `true`.

            Test entities cannot be retrieved when the endpoint is set to live mode, and vice versa.
        :param idempotency_key: A unique key to ensure idempotent requests. This key should be a UUID v4 string.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetConnectBalanceTransferRequest(
            balance_transfer_id=balance_transfer_id,
            testmode=testmode,
            idempotency_key=idempotency_key,
        )

        req = self._build_request(
            method="GET",
            path="/connect/balance-transfers/{balanceTransferId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/hal+json",
            http_headers=http_headers,
            _globals=models.GetConnectBalanceTransferGlobals(
                testmode=self.sdk_configuration.globals.testmode,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            allowed_fields=["organization_access_token", "o_auth"],
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 5000, 2, 7500), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5xx"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get-connect-balance-transfer",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/hal+json"):
            return unmarshal_json_response(
                models.EntityBalanceTransferResponse, http_res
            )
        if utils.match_response(http_res, "404", "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)

    async def get_async(
        self,
        *,
        balance_transfer_id: str,
        testmode: Optional[bool] = None,
        idempotency_key: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.EntityBalanceTransferResponse:
        r"""Get a Connect balance transfer

        Retrieve a single Connect balance transfer object by its ID.

        If set, this operation will use either `organization_access_token` or `o_auth` from the global security.

        :param balance_transfer_id: Provide the ID of the related balance transfer.
        :param testmode: You can enable test mode by setting the `testmode` query parameter to `true`.

            Test entities cannot be retrieved when the endpoint is set to live mode, and vice versa.
        :param idempotency_key: A unique key to ensure idempotent requests. This key should be a UUID v4 string.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetConnectBalanceTransferRequest(
            balance_transfer_id=balance_transfer_id,
            testmode=testmode,
            idempotency_key=idempotency_key,
        )

        req = self._build_request_async(
            method="GET",
            path="/connect/balance-transfers/{balanceTransferId}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/hal+json",
            http_headers=http_headers,
            _globals=models.GetConnectBalanceTransferGlobals(
                testmode=self.sdk_configuration.globals.testmode,
            ),
            security=self.sdk_configuration.security,
            allow_empty_value=None,
            allowed_fields=["organization_access_token", "o_auth"],
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config
            else:
                retries = utils.RetryConfig(
                    "backoff", utils.BackoffStrategy(500, 5000, 2, 7500), True
                )

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["5xx"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                config=self.sdk_configuration,
                base_url=base_url or "",
                operation_id="get-connect-balance-transfer",
                oauth2_scopes=None,
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/hal+json"):
            return unmarshal_json_response(
                models.EntityBalanceTransferResponse, http_res
            )
        if utils.match_response(http_res, "404", "application/hal+json"):
            response_data = unmarshal_json_response(models.ErrorResponseData, http_res)
            raise models.ErrorResponse(response_data, http_res)
        if utils.match_response(http_res, "4XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError("API error occurred", http_res, http_res_text)

        raise models.APIError("Unexpected response received", http_res)



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 7679aeb6b4f9

from __future__ import annotations
from enum import Enum
from mollie import utils


class SettlementRefundStatus(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The refund's status. Settlement refunds always have a status of `refunded`."""

    REFUNDED = "refunded"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: ab7f5a0e11d2

from __future__ import annotations
from enum import Enum
from mollie import utils


class SettlementCaptureStatus(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The capture's status. Settlement captures always have a status of `succeeded`."""

    SUCCEEDED = "succeeded"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 94194c013cae

from __future__ import annotations
from .transfer_scheme_type_response import TransferSchemeTypeResponse
from mollie import models
from mollie.types import BaseModel
from pydantic import field_serializer
from typing_extensions import TypedDict


class TransferSchemeResponseTypedDict(TypedDict):
    r"""The scheme, as requested by the client."""

    type: TransferSchemeTypeResponse
    r"""The transfer scheme to be used for the transfer. The transfer scheme determines the processing time and method of the transfer."""


class TransferSchemeResponse(BaseModel):
    r"""The scheme, as requested by the client."""

    type: TransferSchemeTypeResponse
    r"""The transfer scheme to be used for the transfer. The transfer scheme determines the processing time and method of the transfer."""

    @field_serializer("type")
    def serialize_type(self, value):
        if isinstance(value, str):
            try:
                return models.TransferSchemeTypeResponse(value)
            except ValueError:
                return value
        return value



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: c9427208725d

from __future__ import annotations
from mollie.types import BaseModel
import pydantic
from typing_extensions import Annotated, TypedDict


class TransferPartyAccountTypedDict(TypedDict):
    r"""The bank account details of the party."""

    iban: str
    r"""The IBAN (International Bank Account Number) of the account holder."""


class TransferPartyAccount(BaseModel):
    r"""The bank account details of the party."""

    iban: str
    r"""The IBAN (International Bank Account Number) of the account holder."""


class TransferPartyTypedDict(TypedDict):
    r"""A party involved in the transfer, representing either the debtor (sender) or creditor (recipient).
    Contains the party's name and account details.
    """

    full_name: str
    r"""The full name of the account holder."""
    account: TransferPartyAccountTypedDict
    r"""The bank account details of the party."""


class TransferParty(BaseModel):
    r"""A party involved in the transfer, representing either the debtor (sender) or creditor (recipient).
    Contains the party's name and account details.
    """

    full_name: Annotated[str, pydantic.Field(alias="fullName")]
    r"""The full name of the account holder."""

    account: TransferPartyAccount
    r"""The bank account details of the party."""


try:
    TransferParty.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 994833ff2883

from __future__ import annotations
from .amount import Amount, AmountTypedDict
from .metadata import Metadata, MetadataTypedDict
from .transfer_party import TransferParty, TransferPartyTypedDict
from .transfer_scheme import TransferScheme, TransferSchemeTypedDict
from mollie.types import BaseModel, Nullable, OptionalNullable, UNSET, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing_extensions import Annotated, NotRequired, TypedDict


class TransferRequestTypedDict(TypedDict):
    debtor_iban: str
    r"""The IBAN of the debtor's (sender) Mollie account from which to initiate the transfer."""
    creditor: TransferPartyTypedDict
    r"""A party involved in the transfer, representing either the debtor (sender) or creditor (recipient).
    Contains the party's name and account details.
    """
    amount: AmountTypedDict
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""
    transfer_scheme: TransferSchemeTypedDict
    r"""The scheme, as requested by the client."""
    description: NotRequired[Nullable[str]]
    r"""A short description of the transfer. This will appear on the bank statement of both the debtor and creditor.

    It must begin with an alphanumeric character, followed by any combination of letters, numbers, spaces or special characters `/ - ? : ( ) . , ' + _`.

    Constraints:
    - Cannot be solely a hyphen (-) or colon (:)
    - Cannot contain consecutive hyphens (--)
    - Cannot contain consecutive colons (::)
    """
    metadata: NotRequired[Nullable[MetadataTypedDict]]
    r"""Provide any data you like, for example a string or a JSON object. We will save the data alongside the entity. Whenever
    you fetch the entity with our API, we will also include the metadata. You can use up to approximately 1kB.
    """
    testmode: NotRequired[Nullable[bool]]
    r"""Whether to create the entity in test mode or live mode.

    Most API credentials are specifically created for either live mode or test mode, in which case this parameter must
    not be sent. For organization-level credentials such as OAuth access tokens, you can enable test mode by setting
    `testmode` to `true`.
    """


class TransferRequest(BaseModel):
    debtor_iban: Annotated[str, pydantic.Field(alias="debtorIban")]
    r"""The IBAN of the debtor's (sender) Mollie account from which to initiate the transfer."""

    creditor: TransferParty
    r"""A party involved in the transfer, representing either the debtor (sender) or creditor (recipient).
    Contains the party's name and account details.
    """

    amount: Amount
    r"""In v2 endpoints, monetary amounts are represented as objects with a `currency` and `value` field."""

    transfer_scheme: Annotated[TransferScheme, pydantic.Field(alias="transferScheme")]
    r"""The scheme, as requested by the client."""

    description: OptionalNullable[str] = UNSET
    r"""A short description of the transfer. This will appear on the bank statement of both the debtor and creditor.

    It must begin with an alphanumeric character, followed by any combination of letters, numbers, spaces or special characters `/ - ? : ( ) . , ' + _`.

    Constraints:
    - Cannot be solely a hyphen (-) or colon (:)
    - Cannot contain consecutive hyphens (--)
    - Cannot contain consecutive colons (::)
    """

    metadata: OptionalNullable[Metadata] = UNSET
    r"""Provide any data you like, for example a string or a JSON object. We will save the data alongside the entity. Whenever
    you fetch the entity with our API, we will also include the metadata. You can use up to approximately 1kB.
    """

    testmode: OptionalNullable[bool] = UNSET
    r"""Whether to create the entity in test mode or live mode.

    Most API credentials are specifically created for either live mode or test mode, in which case this parameter must
    not be sent. For organization-level credentials such as OAuth access tokens, you can enable test mode by setting
    `testmode` to `true`.
    """

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["description", "metadata", "testmode"])
        nullable_fields = set(["description", "metadata", "testmode"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k, serialized.get(n))
            is_nullable_and_explicitly_set = (
                k in nullable_fields
                and (self.__pydantic_fields_set__.intersection({n}))  # pylint: disable=no-member
            )

            if val != UNSET_SENTINEL:
                if (
                    val is not None
                    or k not in optional_fields
                    or is_nullable_and_explicitly_set
                ):
                    m[k] = val

        return m


try:
    TransferRequest.model_rebuild()
except NameError:
    pass



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: 5cd8e3d5cfbf

from __future__ import annotations
from enum import Enum
from mollie import utils


class CreditDebitIndicator(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""Indicates whether the transfer is a credit or debit transaction from the perspective of the account holder."""

    CREDIT = "credit"
    DEBIT = "debit"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""
# @generated-id: b405ad468147

from __future__ import annotations
from enum import Enum
from mollie import utils


class TransferStatus(str, Enum, metaclass=utils.OpenEnumMeta):
    r"""The status of the transfer."""

    REQUESTED = "requested"
    INITIATED = "initiated"
    PENDING_REVIEW = "pending-review"
    PROCESSED = "processed"
    FAILED = "failed"
    BLOCKED = "blocked"
    INCOMING_SETTLED = "incoming-settled"
